<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Archive;

use Illuminate\Support\Collection;
use Interop\Container\ContainerInterface;

class MonthlyArchive
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var array|collection アイテムリスト
     */
    protected $items;
    /**
     * @var array
     */
    protected $links;
    /**
     * @var array
     */
    protected $options;
    /**
     * @var string
     */
    protected $year;
    /**
     * @var string|null
     */
    protected $month;
    /**
     * @var string
     */
    protected $firstPath;
    /**
     * @var string
     */
    protected $routeName;
    /**
     * @var int 合計ページ数
     */
    protected $pageCount;
    /**
     * @var int 現在ページインデックス番号
     */
    protected $currentIndex;

    /**
     * @param Collection $items
     * @param array $links
     * @param array $options
     * @param string $year
     * @param string|null $month
     */
    public function __construct(Collection $items, array $links, array $options, string $year, string $month = null)
    {
        $this->items = $items;
        $this->links = $links;
        $this->options = $options;
        $this->year = $year;
        $this->month = $month;
        $this->setPageInfo();
    }

    /**
     * アーカイブ期間内であることを確認
     *
     * @return bool
     */
    public function outOfBounds(): bool
    {
        foreach ($this->links as $link) {
            if ($link['year'] == $this->year && $link['month'] == $this->month) {
                return false;
            }
        }
        return true;
    }

    /**
     * ルート情報をセット
     *
     * @param ContainerInterface $c
     * @param string $routeName
     */
    public function setRouteInfo(ContainerInterface $c, string $routeName)
    {
        $this->c = $c;
        $this->routeName = $routeName;
    }

    /**
     * @param string $path
     */
    public function setFirstPath(string $path)
    {
        $this->firstPath = $path;
    }

    /**
     * ページ情報をセット
     */
    protected function setPageInfo()
    {
        // 合計ページ数をセット
        $this->pageCount = count($this->links);

        // 現在ページのインデックス番号を確認
        foreach ($this->links as $index => $link) {
            if ($link['year'] == $this->year && $link['month'] == $this->month) {
                $this->currentIndex = $index;
            }
        }
    }

    /**
     * アーカイブアイテム取得
     *
     * @return array|collection
     */
    public function getItems()
    {
        return $this->items;
    }

    /**
     * アーカイブリンク取得
     *
     * @return array
     */
    public function getLinks()
    {
        return $this->links;
    }

    /**
     * 現在のアーカイブ日付の文字列を返す
     *
     * @return string
     */
    public function getDateString(): string
    {
        return "{$this->year}-{$this->month}-01";
    }

    /**
     * 前ページがあることを確認
     *
     * @return bool
     */
    public function hasPrev(): bool
    {
        if ($this->currentIndex === null || $this->pageCount === 0) {
            return false;
        }
        return $this->currentIndex > 0;
    }

    /**
     * 前ページのURLを返す
     *
     * @return string
     */
    public function prevUrl(): string
    {
        if ($this->hasPrev()) {
            return $this->url($this->currentIndex - 1);
        }
        return '#';
    }

    /**
     * 次ページがあることを確認
     *
     * @return bool
     */
    public function hasNext(): bool
    {
        if ($this->currentIndex === null || $this->pageCount === 0) {
            return false;
        }
        return $this->currentIndex < ($this->pageCount - 1);
    }

    /**
     * 次ページのURLを返す
     *
     * @return string
     */
    public function nextUrl(): string
    {
        if ($this->hasNext()) {
            return $this->url($this->currentIndex + 1);
        }
        return '#';
    }

    /**
     * 指定年月のURLを返す
     *
     * @param int $index
     * @return string
     */
    public function url(int $index): string
    {
        if (!isset($this->links[$index])) {
            return '#';
        }
        $link = $this->links[$index];
        if ($link['is_index']) {
            // 1ページ目はfirstPathにセットされたパスをそのまま返す
            return $this->firstPath;
        } else {
            return $this->c->router->pathFor($this->routeName, ['year' => $link['year'], 'month' => $link['month']]);
        }
    }

    /**
     * @return string
     */
    public function getYear(): string
    {
        return $this->year;
    }

    /**
     * @return string
     */
    public function getMonth(): string
    {
        return $this->month;
    }

    /**
     * @return array
     */
    public function getOptions(): array
    {
        return $this->options;
    }
}
