<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use MicroEngine\Cms\Models\Module;
use MicroEngine\Cms\Forms\ModuleForm;
use Psr\Http\Message\ResponseInterface;
use Interop\Container\ContainerInterface;
use MicroEngine\Cms\Services\ModuleManager;
use MicroEngine\Core\Controllers\Controller;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Cms\Services\ContentService;

class ModuleController extends Controller
{
    /**
     * @var ModuleManager
     */
    protected $moduleManager;
    /**
     * @var ContentService
     */
    protected $contentService;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        parent::__construct($c);

        $this->contentService = $c->contentService;
        $this->moduleManager = $c->moduleManager;
        // ライセンスチェック
        $this->c->license->hasLicense();
    }

    /**
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $modules = $this->contentService->moduleConfig();
        history_unset('module');

        return $this->view->render(
            $response,
            'cms/views/module/index.twig',
            compact('modules')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function create(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = content_config($type, 'module');
        $form = new ModuleForm($this->c);
        $form->setConfig($config);
        $form->setInitialValues();
        $form->set('edit_mode', 'create');

        return $this->view->render(
            $response,
            'cms/views/module/create.twig',
            compact('type', 'config', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = content_config($type, 'module');
        $form = new ModuleForm($this->c);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $module = $this->moduleManager->create($values);
            $this->flash->addMessage('success', 'モジュールを作成しました。モジュール：' . $config['label']);

            if (history_has('module')) {
                return $response->withRedirect(history('module'));
            }
            return $response->withRedirect($this->c->router->pathFor('admin.module.index'));
        } else {
            return $this->view->render(
                $response,
                'cms/views/module/create.twig',
                compact('type', 'config', 'form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function preview(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = content_config($type, 'module');
        $form = new ModuleForm($this->c);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());
        $site = $this->c->site;
        $site->setPreview(true);

        // タイプ別プレビューテンプレートのパス
        $template = '@site/module/preview/' . $type . '.twig';
        if (!$this->view->getLoader()->exists($template)) {
            // タイプ別のテンプレートが用意されていない場合は汎用テンプレートを指定する。
            $template = 'cms/views/base/preview.twig';
        }

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $module = new Module($values);
            $site->pushContent($module, true);

            return $this->view->render(
                $response,
                $template,
                compact('module', 'config')
            );
        } else {
            return $this->view->render(
                $response,
                'cms/views/base/previewError.twig',
                compact('form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'] ?? null;
        $editMode = $args['edit_mode'] ?? 'edit';
        $revisionId = $args['revision_id'] ?? null;
        if ($revisionId) {
            $module = $this->moduleManager->revision($id, $revisionId);
        } else {
            $module = Module::findOrFail($id);
        }
        $type = $module->type;
        $config = content_config($type, 'module');
        $form = new ModuleForm($this->c);
        $form->setId($args['id']);
        $form->setConfig($config);
        $form->setInputValue($module);
        $form->set('edit_mode', $editMode);
        $form->set('revision_id', $revisionId);

        return $this->view->render(
            $response,
            'cms/views/module/edit.twig',
            compact('type', 'config', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $module = Module::findOrFail($args['id']);
        $type = $module->type;
        $config = content_config($type, 'module');
        $form = new ModuleForm($this->c);
        $form->setId($args['id']);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $this->moduleManager->update($args['id'], $values);
            $this->flash->addMessage('success', 'モジュールを更新しました。モジュール：' . $config['label']);

            if (history_has('module')) {
                return $response->withRedirect(history('module'));
            }
            return $response->withRedirect($this->c->router->pathFor('admin.module.index'));
        } else {
            return $this->view->render(
                $response,
                'cms/views/module/edit.twig',
                compact('type', 'config', 'form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function info(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_set('module', $request);

        $module = Module::findOrFail($args['id']);
        $type = $module->type;
        $config = content_config($type, 'module');
        $revisions = $this->moduleManager->revisionList($module);

        return $this->view->render(
            $response,
            'cms/views/module/info.twig',
            compact('module', 'type', 'config', 'revisions')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'];
        $revisionId = $args['revision_id'] ?? null;

        $module = $this->moduleManager->destroy($id, $revisionId);
        $config = content_config($module->type, 'module');
        $message = ($revisionId) ? '履歴' : 'モジュール';
        $this->flash->addMessage('success', $message . 'を削除しました。モジュール：' . $config['label']);

        if ($revisionId !== null && history_has('module')) {
            return $response->withRedirect(history('module'));
        }
        return $response->withRedirect($this->c->router->pathFor('admin.module.index', ['type' => $module->type]));
    }
}
