<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Forms;

use Carbon\Carbon;
use Rakit\Validation\Valiator;
use MicroEngine\Core\Forms\Form;
use MicroEngine\Site\Services\Site;
use Interop\Container\ContainerInterface;

class ContentForm extends Form
{
    /**
     * @var array
     */
    protected $config;
    /**
     * @var array
     */
    protected $fields;
    /**
     * @var array
     */
    protected $values;
    /**
     * @var array
     */
    protected $labels;
    /**
     * @var array
     */
    protected $rules;
    /**
     * @var Site
     */
    protected $site;
    /**
     * @var bool 下書き保存可能
     */
    protected $canDraft = false;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->site = $c->site;
        $this->setInitialInputs();
    }

    /**
     * バリデーション
     *
     * @return bool
     * @throws \Rakit\Validation\RuleQuashException
     */
    public function validate(): bool
    {
        $validator = $this->getValidator();

        // 標準フィールドをセット
        $this->labels = array_unnest($this->inputs, 'label');
        $this->values = array_unnest($this->inputs, 'value');
        $this->rules = array_unnest($this->inputs, 'rule');

        // カスタムフィールドをセット
        foreach ($this->config['fields'] as $fieldName => $field) {
            if ($field['container'] ?? false) {
                // コンテナ対応
                $this->validateContainerField($fieldName, $field);
            } else {
                // コンテナ外フィールド
                foreach ($field['inputs'] as $inputName => $input) {
                    $dottedName = $fieldName . '-' . $inputName;
                    $this->labels[$dottedName] = $input['label'];
                    $this->values[$dottedName] = $this->fields[$fieldName][$inputName] ?? null;
                    $this->rules[$dottedName] = $input['rule'];
                }
            }
        }
        $validation = $validator->make($this->values, $this->rules);
        $validation->setAliases($this->labels);
        $validation->validate();

        if ($validation->fails()) {
            // handling errors
            $this->errorBag = $validation->errors();

            return false;
        } else {
            $this->errorBag = null;

            return true;
        }
    }

    /**
     * コンテナフィールドのバリデーション
     *
     * @param string $fieldName
     * @param array $field
     */
    protected function validateContainerField(string $fieldName, array $field)
    {
        if (!isset($this->fields[$fieldName])) {
            // 空コンテナの場合はスルー
            return;
        }
        $i = 0;
        foreach ($this->fields[$fieldName] as $index => $row) {
            foreach ($row as $childFieldName => $childField) {
                foreach ($field['fields'][$childFieldName]['inputs'] as $inputName => $input) {
                    $dottedName = $fieldName . '-' . $i . '-' . $childFieldName . '-' . $inputName;
                    $this->labels[$dottedName] = $input['label'];
                    $this->values[$dottedName] = $this->fields[$fieldName][$index][$childFieldName][$inputName] ?? null;
                    if (isset($input['format']) && $input['format'] === 'json') {
                        $this->fields[$fieldName][$index][$childFieldName][$inputName] = json_decode($this->values[$dottedName], true);
                    }
                    $this->rules[$dottedName] = $input['rule'];
                }
            }
            $i++;
        }
    }

    /**
     * 入力値をセット
     *
     * @param array $data
     */
    public function setInputValue($data)
    {
        parent::setInputValue($data);

        if (isset($data['fields'])) {
            $this->fields = $data['fields'];
        } else {
            // フィールドなしコンテンツ対応
            $this->fields = [];
        }
    }

    /**
     * フォームの初期値をセット
     */
    public function setInitialValues()
    {
        $this->fields = [];

        // カスタムフィールドをセット
        foreach ($this->config['fields'] as $fieldName => $field) {
            if ($field['container'] ?? false) {
                // コンテナは初期値はない
                continue;
            }
            // コンテナ外フィールド
            foreach ($field['inputs'] as $inputName => $input) {
                if (isset($input['value'])) {
                    $this->fields[$fieldName][$inputName] = $input['value'];
                }
            }
        }
    }

    /**
     * 入力値を取得
     *
     * @param bool $renumber フィールドの配列の添字を割り当て直すかどうか
     * @return array
     */
    public function getInputValues(bool $renumber = true): array
    {
        // 標準フィールド対応
        $values = array_unnest($this->inputs, 'value');

        // カスタムフィールド対応
        $values['fields'] = [];
        foreach ($this->fields as $fieldName => $field) {
            // コンテナ対応
            if ($this->config['fields'][$fieldName]['container'] ?? false) {
                // コンテナを持つフィールドは
                foreach ($field as $index => $row) {
                    foreach ($row as $childFieldName => $childField) {
                        foreach ($childField as $inputName => $input) {
                            if (!isset($this->config['fields'][$fieldName]['fields'][$childFieldName]['inputs'][$inputName])) {
                                // 設定に定義されていないインプットは削除する
                                unset($this->fields[$fieldName][$index][$childFieldName][$inputName]);
                            }
                        }
                    }
                }
                if ($renumber) {
                    $this->fields[$fieldName] = array_values($this->fields[$fieldName]);
                }
                continue;
            }
            // コンテナ外フィールド対応
            foreach ($field as $inputName => $input) {
                if (!isset($this->config['fields'][$fieldName]['inputs'][$inputName])) {
                    // 設定に無いフィールドは削除する
                    unset($this->fields[$fieldName][$inputName]);
                }
            }
        }
        $values['fields'] = $this->fields;

        return $values;
    }

    /**
     * フィールドID指定でフィールド値を取得
     *
     * @param string $name
     * @param mixed|null $default
     * @return mixed|string
     */
    public function get(string $name, $default = null)
    {
        // カスタムフィールド値を取得
        if (strpos($name, '-') !== false) {
            return $this->getFieldValue($name);
        }
        // 標準フィールド値を取得
        if (array_key_exists($name, $this->inputs)) {
            if (isset($this->inputs[$name]['date_picker'])
                && $this->inputs[$name]['date_picker'] === 'date'
                && !empty($this->inputs[$name]['value'])) {
                // date の場合は年月日のみ表示する
                return Carbon::parse($this->inputs[$name]['value'])->format('Y-m-d');
            }

            return $this->inputs[$name]['value'];
        }
        return $default;
    }

    /**
     * フィールド名指定でフィールド値を取得
     *
     * @param string $name
     * @return mixed|string
     */
    public function field(string $name)
    {
        return $this->getFieldValue($this->site->inputId($name));
    }

    /**
     * コンテナ内の子フィールドを取得
     *
     * @param string $fieldName
     * @param string $parentFieldName
     * @return mixed
     */
    public function getChildField(string $fieldName, string $parentFieldName)
    {
        if (!isset($this->config['fields'][$parentFieldName]['fields'][$fieldName])) {
            return [];
        }
        return $this->config['fields'][$parentFieldName]['fields'][$fieldName];
    }

    /**
     * ラベルテキストを取得
     *
     * @param string $name
     * @return string
     */
    public function label(string $name): string
    {
        if (array_key_exists($name, $this->inputs)) {
            return $this->inputs[$name]['label'];
        }
        return '';
    }

    /**
     * 非表示設定されているかどうかを確認
     *
     * @param string $name
     * @return bool
     */
    public function hidden(string $name): bool
    {
        if (isset($this->inputs[$name]['hidden'])) {
            return (boolean)$this->inputs[$name]['hidden'];
        }
        return false;
    }

    /**
     * コンテンツの設定をセット
     *
     * @param array $config
     */
    public function setConfig(array $config)
    {
        $this->config = $config;

        if (isset($config['inputs']) && is_array($config['inputs'])) {
            $this->inputs = array_replace_recursive($this->inputs, $config['inputs']);
        }
        if (isset($this->inputs['sitemap']) && isset($config['sitemap'])) {
            // sitemap設定があれば初期値としてセットする
            $this->inputs['sitemap']['value'] = $config['sitemap'];
        }
    }

    /**
     * @param  array $fields
     */
    public function setFields(array $fields)
    {
        $this->fields = $fields;
    }

    /**
     * @return array
     */
    public function getFields(): array
    {
        return $this->fields;
    }

    /**
     * フィールドを取得
     *
     * @param string $name
     * @return mixed|null
     */
    public function fieldContainer(string $name)
    {
        return $this->fields[$name] ?? null;
    }

    /**
     * カスタムフィールド値を取得
     *
     * @param string $name
     * @return mixed|string
     */
    protected function getFieldValue(string $name)
    {
        $key = str_replace('-', '.', $name);
        if (array_has($this->fields, $key)) {
            return array_get($this->fields, $key);
        }
        return '';
    }

    /**
     * @return array
     */
    public function getInputs(): array
    {
        return $this->inputs;
    }

    /**
     * selected属性を出力
     *
     * @param string $name
     * @param int|string $value
     * @return string
     */
    public function selected(string $name, $value): string
    {
        if ((string)$this->get($name) === (string)$value) {
            return ' selected="selected"';
        }
        return '';
    }

    /**
     * 下書き保存できるかどうかを返す
     *
     * @return bool
     */
    public function canDraft()
    {
        return $this->canDraft;
    }
}
