<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Managers;

use Carbon\Carbon;
use MicroEngine\Core\Views\View;
use MicroEngine\Cms\Models\Calendar;
use Interop\Container\ContainerInterface;
use MicroEngine\Cms\Models\CalendarRevision;
use MicroEngine\Cms\Services\ContentService;
use MicroEngine\Cms\Calendar\MonthlyCalendar;

class CalendarManager
{
    use RevisionTrait;

    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var string モデルクラス名
     */
    protected $modelClass = 'MicroEngine\Cms\Models\Calendar';
    /**
     * @var string IDカラム名
     */
    protected $modelId = 'calendar_id';
    /**
     * @var string リビジョンクラス名
     */
    protected $revisionClass = 'MicroEngine\Cms\Models\CalendarRevision';
    /**
     * @var string リビジョンテーブル名
     */
    protected $revisionTable = 'calendar_revisions';

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * カレンダー保存
     *
     * @param array $data
     * @return Calendar
     */
    public function create(array $data): Calendar
    {
        // 作成者・更新者にログインユーザーのIDをセットする
        $data['create_user_id'] = $data['update_user_id'] = $this->c->auth->getUserId();

        $calendar = Calendar::create($data);

        // リビジョン保存
        $this->saveRevision($calendar->id, $data);

        // キャッシュをクリアする
        $this->c->htmlCache->clear();

        $this->c->logger->addInfo('カレンダーを作成しました。', [
            'type' => $calendar->type,
            'date' => $calendar->date,
        ]);
        return $calendar;
    }

    /**
     * カレンダー更新
     *
     * @param int $id
     * @param array $data
     * @return Calendar
     */
    public function update(int $id, array $data): Calendar
    {
        // 更新者にログインユーザーのIDをセットする
        $data['update_user_id'] = $this->c->auth->getUserId();

        $calendar = Calendar::withoutGlobalScopes()->findOrFail($id);
        $calendar->fill($data);
        $calendar->save();

        // リビジョン保存
        $this->saveRevision($id, $data);

        // キャッシュをクリアする
        $this->c->htmlCache->clear();

        $this->c->logger->addInfo('カレンダーを更新しました。', [
            'type' => $calendar->type,
            'date' => $calendar->date,
        ]);
        return $calendar;
    }

    /**
     * カレンダー削除
     *
     * @param int $id
     * @param int|null $revisionId
     * @return Calendar
     */
    public function destroy(int $id, int $revisionId = null): Calendar
    {
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($id);
        // 更新者にログインユーザーのIDをセットする
        $calendar->update_user_id = $this->c->auth->getUserId();

        if ($revisionId) {
            // 履歴から再編集している場合は、履歴を削除する。
            $revision = CalendarRevision::findOrFail($revisionId);
            $revision->delete();
            // ログを書き出す
            $this->c->logger->addInfo('カレンダーの履歴を削除しました。', [
                'type' => $calendar->type,
                'date' => $calendar->date,
                'revision_id' => $revisionId,
            ]);
        } else {
            // 履歴から編集していない場合は、本レコードを論理削除する。
            $calendar->visibility = ContentService::VISIBILITY_TRASH;
            $calendar->update();

            // ログを書き出す
            $this->c->logger->addInfo('カレンダーをゴミ箱に移動しました。', [
                'type' => $calendar->type,
                'date' => $calendar->date,
            ]);
        }

        // キャッシュをクリアする
        $this->c->htmlCache->clear();

        return $calendar;
    }

    /**
     * カレンダーをごみ箱から削除
     *
     * @param int $id
     * @return Calendar
     */
    public function forceDestroy(int $id): Calendar
    {
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($id);

        // ゴミ箱から削除する場合は、物理削除を行う
        $calendar->delete();

        // キャッシュをクリアする
        $this->c->htmlCache->clear();

        // ログを書き出す
        $this->c->logger->addInfo('カレンダーをごみ箱から削除しました。', [
            'type' => $calendar->type,
            'date' => $calendar->date,
        ]);
        return $calendar;
    }

    /**
     * ごみ箱を空にする
     *
     * @return int
     */
    public function emptyTrash(): int
    {
        $count = 0;
        // ごみ箱内の全コンテンツに対して処理する
        foreach (Calendar::trashed()->get() as $calendar) {
            // コンテンツ一つずつをごみ箱から削除する
            $this->forceDestroy($calendar->id);
            $count++;
        }
        $this->c->logger->addInfo('カレンダーのごみ箱を空にしました。', [
            'count' => $count,
        ]);
        return $count;
    }

    /**
     * 一括作成
     *
     * @param array $data
     * @return int $created 作成数
     */
    public function bulkCreate(array $data): int
    {
        $created = 0;

        foreach ($data as $row) {
            if (isset($row['fields']) && !is_array($row['fields'])) {
                $row['fields'] = json_decode($row['fields'], true);
            }
            $this->create($row);
            $created++;
        }
        $this->c->logger->addInfo('カレンダーを一括作成しました。', [
            'count' => $created,
        ]);
        return $created;
    }

    /**
     * カレンダー一括コピー
     *
     * @param int $id
     * @param string $start
     * @param string $end
     * @param array $options
     * @return int
     */
    public function bulkCopy(int $id, string $start, string $end, array $options = []): int
    {
        $saved = 0;
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($id);
        $startDt = Carbon::parse($start);
        $endDt = Carbon::parse($end);

        $options += [
            'overwrite' => false,
            'daysOfWeek' => [],
            'days' => [],
            'last_day' => false,
        ];
        $overwrite = $options['overwrite'];
        $filter = (boolean)$options['daysOfWeek'] || $options['days'] || $options['last_day'];

        // startが未来の場合は処理しない
        if ($startDt->gt($endDt)) {
            return $saved;
        }

        // 既存データ取得
        $rows = Calendar::withoutGlobalScopes()->where('type', $calendar->type)
            ->where('date', '>=', $start)
            ->where('date', '<=', $end)
            ->get();
        $calendars = [];
        foreach ($rows as $model) {
            $calendars[$model->date] = $model;
        }

        $data = $calendar->toArray();
        unset($data['id']);
        unset($data['created_at']);
        unset($data['updated_at']);

        while ($startDt->lte($endDt)) {
            $data['date'] = $date = $startDt->format('Y-m-d');

            // 絞り込み条件
            if ($filter && !$this->filterDate($startDt, $options)) {
                // 絞り込み条件が指定されていて、一致するものがない場合はスルー
                $startDt->addDay();
                continue;
            }

            // 既存データの有無確認
            if (isset($calendars[$date])) {
                if ($overwrite) {
                    $this->update($calendars[$date]->id, $data);
                    $saved++;
                }
                // 既存データがあり、上書き設定ではない場合はスルー
                $startDt->addDay();
                continue;
            }

            // データ登録
            $this->create($data);
            $saved++;
            $startDt->addDay();
        }
        $this->c->logger->addInfo('カレンダーを一括複製しました。', [
            'type' => $calendar->type,
            'count' => $saved,
        ]);
        return $saved;
    }

    /**
     * カレンダー一括削除
     *
     * @param int $id
     * @param string $start
     * @param string $end
     * @param array $options
     * @return int
     */
    public function bulkDestroy(int $id, string $start, string $end, array $options = []): int
    {
        $deleted = 0;
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($id);
        $startDt = Carbon::parse($start);
        $endDt = Carbon::parse($end);

        $options += [
            'daysOfWeek' => [],
            'days' => [],
            'last_day' => false,
        ];
        $filter = (boolean)$options['daysOfWeek'] || $options['days'] || $options['last_day'];

        // startが未来の場合は処理しない
        if ($startDt->gt($endDt)) {
            return $deleted;
        }

        // 既存データ取得
        $rows = Calendar::withoutGlobalScopes()->where('type', $calendar->type)
            ->where('date', '>=', $start)
            ->where('date', '<=', $end)
            ->get();
        $calendars = [];
        foreach ($rows as $model) {
            $calendars[$model->date] = $model;
        }

        while ($startDt->lte($endDt)) {
            $date = $startDt->format('Y-m-d');

            // 絞り込み条件
            if ($filter && !$this->filterDate($startDt, $options)) {
                // 絞り込み条件が指定されていて、一致するものがない場合はスルー
                $startDt->addDay();
                continue;
            }

            // 既存データの有無確認
            if (isset($calendars[$date])) {
                // データ削除
                $this->destroy($calendars[$date]->id);
                $deleted++;
            }
            $startDt->addDay();
        }
        $this->c->logger->addInfo('カレンダーを一括削除しました。', [
            'type' => $calendar->type,
            'count' => $deleted,
        ]);
        return $deleted;
    }

    /**
     * 月カレンダー表示
     *
     * @param string $type
     * @param string|null $date
     * @param int $months
     * @return string
     * @throws \Twig_Error_Loader
     * @throws \Twig_Error_Runtime
     * @throws \Twig_Error_Syntax
     */
    public function monthlyCalendar(string $type, string $date = null, $months = 1): string
    {
        $calendar = $this->monthlyCalendarData($type, $date, $months);

        /** @var View $view */
        $view = $this->c->view;

        $template = '@site/calendar/monthly/' . $type . '.twig';
        if (!$view->getLoader()->exists($template)) {
            $template = 'cms/views/calendar/monthlyCalendar.twig';
        }

        return $view->getEnvironment()->render(
            $template,
            compact('calendar')
        );
    }

    /**
     * 月カレンダーデータ取得
     *
     * @param string $type
     * @param string|null $date
     * @param int $months
     * @return MonthlyCalendar
     * @throws \Exception
     */
    public function monthlyCalendarData(string $type, string $date = null, $months = 1): MonthlyCalendar
    {
        $calendar = new MonthlyCalendar();
        $calendar->pull($type, $date, $months);

        return $calendar;
    }

    /**
     * 日付の絞り込み
     *
     * @param Carbon $dt
     * @param array $options
     * @return bool
     */
    protected function filterDate(Carbon $dt, array $options): bool
    {
        if (isset($options['daysOfWeek'][$dt->dayOfWeek])) {
            return true;
        }
        if (isset($options['days'][$dt->day])) {
            return true;
        }
        if ($options['last_day']) {
            if ($dt->day === (int)$dt->copy()->endOfMonth()->format('j')) {
                return true;
            }
        }
        return false;
    }
}
