<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Support;

use GlobIterator;
use Slim\Http\UploadedFile;
use MicroEngine\Core\Exception\MeException;

class FileSystem
{
    /**
     * 除外ファイル
     */
    const EXCLUDE_FILES = [
        '.',
        '..',
        '.DS_Store',
        'Thumbs.db',
    ];
    /**
     * @var string このクラスで扱う最上位ディレクトリを指定する
     */
    protected $root;
    /**
     * @var array
     */
    protected $config;

    /**
     * @param string $root
     */
    public function __construct(string $root = '/')
    {
        $this->root = end_slash($root);
        // ルートディレクトリは正規化する。
        $realPath = realpath($this->root);
        if ($realPath !== false) {
            $this->root = end_slash($realPath);
        }

        $this->config = config('settings.filesystem');
        $this->config += [
            'file_permission' => null,
            'dir_permission' => null,
            'file_group' => null,
            'dir_group' => null,
        ];
    }

    /**
     * パターンにマッチするパス名を探す
     *
     * @param string $path
     * @param int $flags
     * @return GlobIterator
     * @throws MeException
     */
    public function glob(string $path, int $flags = null): GlobIterator
    {
        return new GlobIterator($this->fullPath($path), $flags);
    }

    /**
     * パターンにマッチするパスのファイルのみを取得する
     *
     * @param string $path
     * @param int|null $flags
     * @return array
     * @throws MeException
     */
    public function globFiles(string $path, int $flags = null): array
    {
        $files = [];

        $iterator = new GlobIterator($this->fullPath($path), $flags);
        foreach ($iterator as $item) {
            if ($item->isDir()) {
                continue;
            }
            $files[] = $item->getFilename();
        }
        return $files;
    }

    /**
     * ディレクトリ内のファイル一覧を取得
     *
     * @param string $dir
     * @param array $excludeFiles
     * @return array
     * @throws MeException
     */
    public function dirs(string $dir = '/', array $excludeFiles = []): array
    {
        $files = [];
        $excludeFiles = array_merge($excludeFiles, self::EXCLUDE_FILES);

        $items = array_diff(scandir($this->fullPath($dir)), $excludeFiles);
        foreach ($items as $item) {
            $path = $dir . $item;
            $files[$item] = $this->fileStatus($path);
        }
        array_multisort(array_column($files, 'is_dir'), SORT_DESC, $files);

        return $files;
    }

    /**
     * ファイルステータスを取得
     *
     * @param string $path
     * @return array
     * @throws MeException
     */
    public function fileStatus(string $path): array
    {
        $fullPath = $this->fullPath($path);

        $status = [];
        $status['path'] = $path;
        $status['extension'] = pathinfo($path, PATHINFO_EXTENSION);
        $status['is_dir'] = is_dir($fullPath);
        $status['size'] = filesize($fullPath);
        $status['updated_at'] = filemtime($fullPath);
        $status['created_at'] = filectime($fullPath);

        return $status;
    }

    /**
     * ディレクトリ作成
     *
     * @param string $dir
     * @return bool
     * @throws MeException
     */
    public function makeDir(string $dir): bool
    {
        if ($dir === '.') {
            // ディレクトリを含まないパスからディレクトリを取り出そうとすると . が返されることがあり、それが引数として渡される。
            // その場合は true を返す。
            return true;
        }

        // スラッシュ区切りでディレクトリを分解する。
        $dirs = explode('/', ltrim($dir, '/'));
        $parentPath = '';
        foreach ($dirs as $parentDir) {
            $parentPath .= $parentDir . '/';
            if (!$this->has($parentPath)) {
                // ディレクトリがなければ作成する。
                $result = mkdir($this->fullPath($parentPath));
                if ($result === false) {
                    throw new MeException('ディレクトリの作成に失敗しました。ディレクトリ：' . $parentPath);
                }
                // ディレクトリのパーミッション・モードをセットする
                $this->setDirMode($parentPath);
            }
        }
        return true;
    }

    /**
     * このroot内の全ファイルとディレクトリを削除して空にする。
     */
    public function emptyInsideRoot()
    {
        $files = $this->dirs();
        foreach ($files as $file) {
            $this->delete($file['path']);
        }
    }

    /**
     * ファイル・ディレクトリ一括削除
     *
     * @param array $paths
     * @return bool
     * @throws MeException
     */
    public function deleteFiles(array $paths): bool
    {
        foreach ($paths as $path) {
            $this->delete($path);
        }
        return true;
    }

    /**
     * ファイル・ディレクトリ削除
     *
     * @param string $path
     * @return bool
     * @throws MeException
     */
    public function delete(string $path): bool
    {
        if ($this->isDir($path)) {
            $this->deleteDir($path);
        } elseif ($this->has($path)) {
            return unlink($this->fullPath($path));
        }
        return true;
    }

    /**
     * ディレクトリ削除
     *
     * @param string $dir
     * @return bool
     * @throws MeException
     */
    public function deleteDir(string $dir): bool
    {
        if (!$this->has($dir)) {
            return true;
        }

        $dir = end_slash($this->fullPath($dir));

        $iterator = new \RecursiveIteratorIterator(
            new \RecursiveDirectoryIterator($dir, \FilesystemIterator::SKIP_DOTS),
            \RecursiveIteratorIterator::CHILD_FIRST
        );
        foreach ($iterator as $filename => $fileInfo) {
            if ($fileInfo->isDir()) {
                rmdir($filename);
            } else {
                unlink($filename);
            }
        }
        // 最後に最初に指定したディレクトリを削除する
        rmdir($dir);

        return true;
    }

    /**
     * アップロードファイルを移動する
     *
     * @param string $destination
     * @param UploadedFile $uploadedFile
     * @throws MeException
     */
    public function moveUploadedFile(string $destination, UploadedFile $uploadedFile)
    {
        // 親ディレクトリを作成する
        $this->makeParentDir($destination);
        // アップロードファイルを移動する
        $uploadedFile->moveTo($this->fullPath($destination));
        // パーミッション設定
        $this->setFileMode($destination);
    }

    /**
     * ファイルの存在確認
     *
     * @param string $path
     * @return bool
     * @throws MeException
     */
    public function has(string $path): bool
    {
        return file_exists($this->fullPath($path));
    }

    /**
     * ファイルの存在を確認してなければ例外とする
     *
     * @param string $path
     * @throws MeException
     */
    public function hasOrFail(string $path)
    {
        if (!file_exists($this->fullPath($path))) {
            throw new MeException('指定されたファイルが見つかりません。ファイル：' . $path);
        }
    }

    /**
     * ファイルの内容を取得
     *
     * @param string $path
     * @param bool $checkExists
     * @return bool|string
     * @throws MeException
     */
    public function get(string $path, $checkExists = true)
    {
        if ($checkExists && !$this->has($path)) {
            throw new MeException('指定されたファイルが見つかりません。ファイル：' . $path);
        }
        return file_get_contents($this->fullPath($path));
    }

    /**
     * ファイルの内容を保存
     *
     * @param string $path
     * @param mixed $content
     * @param bool $makeParentDir
     * @return bool|int
     * @throws MeException
     */
    public function put(string $path, $content, $makeParentDir = true)
    {
        if ($makeParentDir) {
            $this->makeParentDir($path);
        }
        $result = file_put_contents($this->fullPath($path), $content);
        if ($result === false) {
            throw new MeException('ファイルの書き込みに失敗しました。ファイル：' . $path);
        }
        $this->setFileMode($path);

        return $result;
    }

    /**
     * ファイルを複製
     *
     * @param string $source
     * @param string $destination
     * @return bool
     * @throws MeException
     */
    public function copy(string $source, string $destination): bool
    {
        if (!$this->has($source)) {
            return false;
        }
        $result = copy($this->fullPath($source), $this->fullPath($destination));
        if ($result === false) {
            throw new MeException("ファイルのコピーに失敗しました。コピー元：{$source}、コピー先：{$destination}");
        }
        // コピーしたファイルのパーミッションを調整する
        $this->setFileMode($destination);

        return true;
    }

    /**
     * ファイルのパーミッション・グループを設定する
     *
     * @param string $path
     * @throws MeException
     */
    public function setFileMode(string $path)
    {
        $fullPath = $this->fullPath($path);
        if ($this->config['file_permission']) {
            $result = chmod($fullPath, $this->config['file_permission']);
            if ($result === false) {
                throw new MeException('ファイルパーミッションの設定に失敗しました。ファイル：' . $path);
            }
        }
        if ($this->config['file_group']) {
            $result = chgrp($fullPath, $this->config['file_group']);
            if ($result === false) {
                throw new MeException('ファイルグループの設定に失敗しました。ファイル：' . $path);
            }
        }
    }

    /**
     * ディレクトリのパーミッション・グループを設定する
     *
     * @param string $path
     * @throws MeException
     */
    public function setDirMode(string $path)
    {
        $fullPath = $this->fullPath($path);
        if ($this->config['dir_permission']) {
            $result = chmod($fullPath, $this->config['dir_permission']);
            if ($result === false) {
                throw new MeException('ディレクトリパーミッションの設定に失敗しました。ディレクトリ：' . $path);
            }
        }
        if ($this->config['file_group']) {
            $result = chgrp($fullPath, $this->config['dir_group']);
            if ($result === false) {
                throw new MeException('ディレクトリグループの設定に失敗しました。ディレクトリ：' . $path);
            }
        }
    }

    /**
     * md5チェックサム値を取得
     *
     * @param string $path
     * @return bool|string
     * @throws MeException
     */
    public function md5(string $path)
    {
        if ($this->has($path)) {
            return md5_file($this->fullPath($path));
        }
        return false;
    }

    /**
     * ファイルの末尾の内容を取得
     *
     * @param string $path
     * @param int $numberLines
     * @return string
     * @throws MeException
     */
    public function tail(string $path, int $numberLines = 10): string
    {
        if (!$this->has($path)) {
            return '';
        }
        $lines = file($this->fullPath($path));
        $total = count($lines);
        $offset = $total - $numberLines;
        if ($offset < 0) {
            $offset = 0;
        }
        $sliced = array_slice($lines, $offset);

        return implode($sliced);
    }

    /**
     * 指定したパスのファイルシステム上のフルパスを返す
     *
     * @param string $path
     * @return string
     * @throws MeException
     */
    public function fullPath(string $path): string
    {
        $fullPath = $this->root . ltrim($path, '/');
        // ../ などを正規化した realPath に変換する
        $realPath = realpath($fullPath);

        // rootは / 終わりなので、$realPath も / 終わりにして比較する。
        if ($realPath !== false && strpos($realPath . '/', $this->root) !== 0) {
            // realPathにファイルが存在する場合は、ルート内であることを確認する
            // ルート外であってもファイルが存在しない場合は例外にはならない。
            throw new MeException('対象ディレクトリ外のファイルは利用できません。指定ファイル：' . $path);
        }

        return $fullPath;
    }

    /**
     * ファイルシステム上のフルパスを一括取得する
     *
     * @param array $paths
     * @param array $excludePaths
     * @return array
     * @throws MeException
     */
    public function fullPaths(array $paths, array $excludePaths = []): array
    {
        $files = [];

        // 除外ファイルのフルパスの配列を作る
        $excludeFullPaths = [];
        foreach ($excludePaths as $excludePath) {
            $excludeFullPaths[] = $this->fullPath($excludePath);
        }

        foreach ($paths as $path) {
            if ($this->isDir($path)) {
                $iterator = new \RecursiveIteratorIterator(new \RecursiveDirectoryIterator($this->fullPath($path),
                    \FilesystemIterator::SKIP_DOTS | \FilesystemIterator::UNIX_PATHS
                ));
                foreach ($iterator as $fullPath => $info) {
                    if (in_array($info->getFilename(), self::EXCLUDE_FILES)) {
                        // 除外ファイル名に一致した場合はスルーする
                        continue;
                    }
                    foreach ($excludeFullPaths as $excludeFullPath) {
                        if (strpos($fullPath, $excludeFullPath) !== false) {
                            // 除外フルパスと一致したら除外する
                            continue 2;
                        }
                    }
                    $files[] = $fullPath;
                }
            } else {
                $files[] = $this->fullPath($path);
            }
        }
        return $files;
    }

    /**
     * ディレクトリかどうかを調べる
     *
     * @param string $path
     * @return bool
     * @throws MeException
     */
    protected function isDir(string $path): bool
    {
        return is_dir($this->fullPath($path));
    }

    /**
     * 親ディレクトリ作成
     * 指定されたパスの親ディレクトリを作成する。
     *
     * @param string $path
     * @return bool
     * @throws MeException
     */
    protected function makeParentDir(string $path): bool
    {
        // ルートディレクトリを確認してなければ作成する
        $this->makeRootDir();

        // パスの親ディレクトリを取り出す
        $dir = dirname($path);

        if ($dir === '.') {
            // ディレクトリが含まれていない場合は、"." が返される。その場合は、trueを返す。
            return true;
        }

        return $this->makeDir($dir);
    }

    /**
     * ルートディレクトリ作成
     * このクラスのルートディレクトリの有無を確認して存在しない場合は作成する
     *
     * @throws MeException
     */
    protected function makeRootDir()
    {
        if ($this->has('')) {
            // ルートディレクトリが存在する場合はスルー
            return;
        }

        // 末尾の / は削除するが、先頭の / はあえて残す。windows環境ではパスの先頭に / をつけられないため。
        $baseDirs = explode('/', rtrim($this->root, '/'));
        $targetDirs = [];
        // ディレクトリが存在することを末端から確認する
        do {
            $poppedDir = array_pop($baseDirs);
            $targetDirs[] = $poppedDir;
            $path = implode('/', $baseDirs);
            // array_pop は空配列や配列以外の場合は null を返すので、それを確認するとループを終了する
        } while ($poppedDir !== null && !file_exists($path));

        // このクラスのルートパスを一時的に入れ替えて処理する
        $originalRoot = $this->root;
        $this->root = end_slash($path);

        $targetPath = implode('/', $targetDirs);
        // オリジナルのroot ディレクトリを作成する
        $this->makeDir($targetPath);
        // rootプロパティを元に戻す
        $this->root = $originalRoot;
    }
}
