<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Support;

use MicroEngine\Admin\Models\MeSystem;
use Interop\Container\ContainerInterface;
use Illuminate\Database\Capsule\Manager as DB;

class SystemInfo
{
    /**
     * @var ContainerInterface
     */
    protected $c;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * ソフトウェア情報を取得する
     *
     * @return array
     */
    public function software(): array
    {
        $info = [];

        $info['name'] = config('settings.name');
        $info['version'] = config('settings.version');
        $info['schema_version'] = config('settings.schema_version');
        $info['install_date'] = MeSystem::where('name', 'install_date')->value('value');
        $info['debug'] = config('settings.debug', 0);
        $info['displayErrorDetails'] = config('settings.displayErrorDetails', 0);

        return $info;
    }

    /**
     * サーバー情報を取得する
     *
     * @return array
     */
    public function server(): array
    {
        $info = [];

        $info['server_host'] = $_SERVER['HTTP_HOST'];
        // システムディレクトリのディスク容量を確認
        $systemTotal = disk_total_space(system_root());
        $systemFree = disk_free_space(system_root());
        $systemUsed = $systemTotal - $systemFree;
        $info['system_total'] = $systemTotal;
        $info['system_free'] = $systemFree;
        $info['system_used'] = $systemUsed;
        // ドキュメントルートのディスク容量を確認
        $docTotal = disk_total_space(document_root());
        $docFree = disk_free_space(document_root());
        $docUsed = $docTotal - $docFree;
        if ($docTotal !== $systemTotal || $docFree !== $systemFree) {
            // パーテーションが異なる場合は、ドキュメントルートの容量もセットする
            $info['doc_total'] = $docTotal;
            $info['doc_free'] = $docFree;
            $info['doc_used'] = $docUsed;
        } else {
            // パーティションが同じ場合は null をセットする
            $info['doc_total'] = null;
            $info['doc_free'] = null;
            $info['doc_used'] = null;
        }
        return $info;
    }

    /**
     * データベース情報を取得する
     *
     * @return array
     */
    public function database(): array
    {
        $info = [];

        $connection = config('db.default', 'mysql');
        $info += config('db.connections.' . $connection);
        unset($info['username']);
        unset($info['password']);

        // DBMSのバージョンコメントを取得
        $info['dbms_comment'] = $this->dbmsComment();

        // DBMSのバージョンを取得
        $info['dbms_version'] = $this->dbmsVersion();

        return $info;
    }

    /**
     * データベース情報を取得する
     *
     * @return array
     */
    public function php(): array
    {
        $info = [];

        $info['version'] = PHP_VERSION;
        // PHP拡張の有無を確認
        $info['extensions'] = $this->c->installService->checkPhpExtensions();
        // PHP設定を確認
        $info['memory_limit'] = ini_get('memory_limit');
        $info['post_max_size'] = ini_get('post_max_size');
        $info['upload_max_filesize'] = ini_get('upload_max_filesize');

        return $info;
    }

    /**
     * クライアント情報を取得する
     *
     * @return array
     */
    public function client(): array
    {
        $info = [];

        $info['ip'] = $_SERVER['REMOTE_ADDR'];
        $info['ua'] = $_SERVER['HTTP_USER_AGENT'];

        return $info;
    }

    /**
     * ライブラリ情報を取得する
     *
     * @return array
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function libs(): array
    {
        $fs = new FileSystem(system_root());

        $composerLock = json_decode($fs->get('composer.lock'), true);
        $info = $composerLock['packages'];

        $libs = json_decode($fs->get('microengine/Core/resources/libs.json'), true);
        $info = array_merge($libs, $info);

        return $info;
    }

    /**
     * phpinfo のHTMLを取得する
     *
     * @return string
     */
    public function phpinfo(): string
    {
        //出力バッファリング開始
        ob_start();
        phpinfo();
        $phpinfo = ob_get_contents();
        //バッファを削除
        ob_end_clean();

        return $phpinfo;
    }

    /**
     * mysqlのversion_comment変数の値を取得
     *
     * @return string
     */
    public function dbmsComment(): string
    {
        // DBMSのバージョンコメントを取得
        $versionComment = DB::select('SELECT @@GLOBAL.version_comment as version_comment;');
        $dbmsComment = $versionComment[0]->version_comment ?? '';

        return $dbmsComment;
    }

    /**
     * mysqlのversion変数の値を取得
     *
     * @return string
     */
    public function dbmsVersion(): string
    {
        // DBMSのバージョンを取得
        $version = DB::select('SELECT @@GLOBAL.version as version;');
        $dbmsVersion = $version[0]->version ?? '';

        return $dbmsVersion;
    }
}
