<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Install\Services;

use MicroEngine\Admin\Models\MeSystem;
use Interop\Container\ContainerInterface;

class UpgradeService
{
    /**
     * @var ContainerInterface
     */
    protected $c;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * アップグレードの必要可否判定
     *
     * @return bool
     */
    public function needsUpgrade(): bool
    {
        // DBのスキーマバージョン値を取得
        $dbVersion = $this->dbVersion();
        // ソースコード側のスキーマバージョン値を取得
        $codeVersion = config('settings.schema_version');

        if (!is_numeric($dbVersion) || !is_numeric($codeVersion)) {
            // 数値のバージョンを取得できない場合は false を返す
            return false;
        }
        if ($dbVersion < $codeVersion) {
            // ソースコードを置き換えたことで、DB側の値よりソースコード側の値が大きくなった場合はアップグレードが必要
            return true;
        }
        return false;
    }

    /**
     * DBのスキーマバージョン値を取得
     *
     * @return int
     */
    public function dbVersion(): int
    {
        return (int)MeSystem::where('name', 'schema_version')->value('value');
    }

    /**
     * アップグレード処理
     *
     * @param $from
     * @param $to
     */
    public function upgrade($from, $to)
    {
        // 現在のバージョン（$from）をインクリメントして次のバージョンへのアップグレード処理を行う
        for ($v = $from + 1; $v <= $to; $v++) {
            // 該当バージョンにアップグレードするためのオブジェクトを作成
            $class = '\MicroEngine\Install\Upgrades\Upgrade' . $v;
            $upgrade = new $class($this->c);
            // アップグレード実行
            $upgrade();
        }
        // DBのスキーマバージョンを上げる
        $this->c->meSystemManager->updateByName('schema_version', $to);
        // ログを出力する
        $this->c->logger->addNotice('アップグレードに成功しました。', ['旧バージョン' => $from, '新バージョン' => $to]);
    }
}
