<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Controllers;

use MicroEngine\Cms\Models\Page;
use Slim\Exception\NotFoundException;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Controllers\Action;
use Psr\Http\Message\ServerRequestInterface;

class RoutingAction extends Action
{
    /**
     * ルーティング先を判断するアクション
     *
     * @param  ServerRequestInterface $request
     * @param  ResponseInterface $response
     * @param  array $args
     * @return ResponseInterface
     * @throws NotFoundException
     */
    public function __invoke(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // DBに登録されたページを確認
        $path = $args['path'];

        // path が indexファイル名で終了している場合
        if (ends_with($path, '/' . INDEX_FILENAME . HTML_EXTENSION)) {
            // ファイル名を除いたパスを作成
            $path = end_slash(dirname($path));
        }

        $page = Page::where('path', $path)->first();

        if ($page === null) {
            // ページが無ければ、スタティックアクションを確認
            return $this->StaticAction($request, $response, $args);
        }

        // アクションを取得
        $action = config('pages.' . $page->type . '.action');

        // アクション毎の処理
        if ($action === 'pagination') {
            $pagination = new PaginationAction($this->c);
            return $pagination($request, $response, $args, $page);
        } elseif ($action === 'search') {
            $search = new SearchAction($this->c);
            return $search($request, $response, $args, $page);
        } elseif ($action === 'archive') {
            $archive = new ArchiveAction($this->c);
            return $archive($request, $response, $args, $page);
        } elseif ($action === 'calendar') {
            $calendar = new CalendarAction($this->c);
            return $calendar($request, $response, $args, $page);
        }

        return $this->PageAction($request, $response, $args, $page);
    }

    /**
     * 標準のページアクション
     *
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @param Page $page
     * @return ResponseInterface
     */
    protected function PageAction(ServerRequestInterface $request, ResponseInterface $response, array $args, Page $page): ResponseInterface
    {
        $view = $this->c->view;
        $site = $this->c->site;

        $site->pushContent($page, true);

        return $view->render(
            $response,
            $page->getTemplatePath(),
            compact('page')
        );
    }

    /**
     * テンプレートを直接呼び出すアクション
     *
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws NotFoundException
     */
    protected function StaticAction(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $view = $this->c->view;

        $path = $args['path'];
        if (ends_with($path, '/')) {
            // パスがスラッシュで終わっている場合はインデックスファイル名を追加して、/ でも表示できるようにする
            $path .= INDEX_FILENAME . HTML_EXTENSION;
        }

        $twigPath = $path . '.twig';
        if ($view->getLoader()->exists('@site/static/' . $twigPath)) {
            return $view->render(
                $response,
                '@site/static/' . $twigPath,
                compact('site')
            );
        } else {
            throw new NotFoundException($request, $response);
        }
    }
}
