<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Services;

use MicroEngine\Core\Support\FileSystem;
use Interop\Container\ContainerInterface;

class HtmlCache
{
    /**
     * 有効期限ファイル
     */
    const EXPIRATION_DATE_FILE = 'expiration_date.txt';
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var FileSystem
     */
    protected $fileSystem;
    /**
     * @var bool このリクエスト内でキャッシュクリアを実行済みかどうか
     */
    protected $cleared = false;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->fileSystem = new FileSystem(system_root('storage/cache/html'));
    }

    /**
     * キャッシュファイルの存在を確認する
     *
     * @param string $path
     * @return bool
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function has(string $path): bool
    {
        $cachePath = $this->convertCachePath($path);

        return $this->fileSystem->has($cachePath);
    }

    /**
     * キャッシュファイルを保存する
     *
     * @param string $path
     * @param string $content
     * @return bool|int
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function put(string $path, string $content)
    {
        $cachePath = $this->convertCachePath($path);

        return $this->fileSystem->put($cachePath, $content);
    }

    /**
     * キャッシュディレクトリ内を空にする
     */
    public function clear()
    {
        if ($this->cleared) {
            // キャッシュクリア済みの場合は実行しない
            return;
        }
        // キャッシュディレクトリ内を空にする
        $this->fileSystem->emptyInsideRoot();
        // クリア済みフラグを立てる
        $this->cleared = true;
    }

    /**
     * 有効期限切れ確認
     *
     * @return string
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function checkExpired(): string
    {
        // 有効期限ファイルの有無を確認
        if ($this->fileSystem->has(self::EXPIRATION_DATE_FILE)) {
            $expirationUnixtime = $this->fileSystem->get(self::EXPIRATION_DATE_FILE);
        } else {
            // 有効期限ファイルが無い場合は期限を確認して保存する
            $expirationUnixtime = $this->saveExpirationDate();
        }

        // unixtimeがあり、その値が過去の場合は有効期限が切れていると判断
        // 期限が１つも設定されていない場合は、unixtimeが空になる。
        if ($expirationUnixtime && $expirationUnixtime <= time()) {
            // 有効期限を過ぎていれば、キャッシュをクリアする
            $this->clear();
            // 新たに有効期限を確認して保存する
            $expirationUnixtime = $this->saveExpirationDate();
        }

        return $expirationUnixtime;
    }

    /**
     * キャッシュ有効期限日時を返す
     *
     * @return string
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function expirationDate(): string
    {
        $expirationDate = '';

        $expirationUnixtime = $this->checkExpired();
        if ($expirationUnixtime) {
            $expirationDate = date('Y/m/d H:i:s', $expirationUnixtime);
        }
        return $expirationDate;
    }

    /**
     * キャッシュ作成日時を返す
     * 有効期限ファイル作成日時をキャッシュ作成日時とする。
     *
     * @return string
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function createdAt(): string
    {
        $createdAt = '';

        // 有効期限ファイルの有無を確認
        if ($this->fileSystem->has(self::EXPIRATION_DATE_FILE)) {
            $status = $this->fileSystem->fileStatus(self::EXPIRATION_DATE_FILE);
            $createdAt = date('Y/m/d H:i:s', $status['created_at']);
        }
        return $createdAt;
    }

    /**
     * キャッシュ管理機能が有効かどうか？
     * キャッシュ管理の全体設定と、サイトサービス内の現在コンテンツのコンテンツタイプの設定も確認する。
     *
     * @return bool
     */
    public function isEnabled(): bool
    {
        $htmlCache = config('cms.html_cache', true);
        if (!$htmlCache) {
            // HTMLキャッシュ設定が無効の場合
            return false;
        }

        $type = $this->c->site->getType() ?? '';
        if (!strlen($type)) {
            // タイプを取れない場合は、有効と判断する
            return true;
        }
        $config = config('pages.' . $type);
        // 特定のアクションはキャッシュは無効とする
        $action = $config['action'] ?? null;
        if ($action === 'search') {
            return false;
        }
        // キャッシュ設定を取得。未定義の場合は有効とする。
        $pageCache = $config['cache'] ?? true;

        return (boolean)$pageCache;
    }

    /**
     * 有効期限ファイルを保存する
     *
     * @return int|string
     * @throws \MicroEngine\Core\Exception\MeException
     */
    protected function saveExpirationDate(): string
    {
        // 最も近い変更日時のunixtimeを取得して、それを有効期限タイムスタンプとする
        $expirationUnixtime = $this->c->contentService->closestChangeUnixtime();
        // ファイルを保存する
        $this->fileSystem->put(self::EXPIRATION_DATE_FILE, $expirationUnixtime);

        return $expirationUnixtime;
    }

    /**
     * リクエストパスをキャッシュファイルのパスに変換する
     *
     * @param string $path
     * @return string
     */
    protected function convertCachePath(string $path): string
    {
        if (substr($path, -1) === '/') {
            // スラッシュで終わっているときは、インデックスファイル名を追加する
            $path .= INDEX_FILENAME . HTML_EXTENSION;
        }
        return $path;
    }
}
